/**
* Power Management Plugin 0.2.2
* -----------------------------
* Coded by 3r Ch4mp1
* 
* Parts of code are from Anissian's PRX tutorial samples 
* 
* Uses Dark_Alex 3.71 SDK
* 
* blit.c in contrib directory is adapted from the one in CFW Extender
*
* Original blit.c is from Booster's Devhook SDK
*
* Hope this crap can be useful to someone, I did only to learn and enjoy
* with C/C++
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* The files under the contrib, include and lib directories are not licensed under
* the GNU General Public License version 2. They are licensed under their own terms
*/

#include <pspsdk.h>
#include <pspkernel.h>
#include <pspctrl.h>
#include <pspsystimer.h>
#include <psppower.h>
#include <pspdisplay.h>
#include <pspdisplay_kernel.h>
#include <psprtc.h>
#include "contrib/blit.h"

#define KERNEL_MODE 0x1000

//Button Pressed defines

#define RPRESSED pad.Buttons & PSP_CTRL_RTRIGGER
#define LPRESSED pad.Buttons & PSP_CTRL_LTRIGGER
#define STARTPRESSED pad.Buttons & PSP_CTRL_START
#define SELECTPRESSED pad.Buttons & PSP_CTRL_SELECT
#define NOTEPRESSED pad.Buttons & PSP_CTRL_NOTE
#define SQUAREPRESSED pad.Buttons & PSP_CTRL_SQUARE

//VSH Message colours defines

#define FGCOLOR 0xFFFFFF //Foregorund color is pure white
#define BGCOLOR 0x000000 //Background color is pure black

//PRX Specific stuff
PSP_MODULE_INFO("PowerMgr", KERNEL_MODE, 0, 22);
PSP_MAIN_THREAD_ATTR(0);

/* 
* These are used to access directly to a function
* available in a library, without calling them from
* their include file, this is done with this functions
* to avoid compile-time errors
*/
extern "C"{
int sceDisplayEnable(void);
int sceDisplayDisable(void);
void sctrlHENSetSpeed(int cpu, int bus);
}

SceCtrlData pad; // Structure which represents the PSP controls, we'll use it to interact with the PSP

//
//Music Mode related variables
//

bool isMusicMode = false; // Boolean which tells us if we put the PSP in Music Mode

int clockFrequency;
int busFrequency;

//
//Clock & Bus Speed defines
//

#define MUSIC_MODE_CLKSPD 60 
#define MUSIC_MODE_BUSSPD 30  

//
//PowerLock related variables 
//

bool isSwitchLocked = false;

//
//IdleControl related variables
//

bool isIdleDeactivated = false;

//
//BlackScreenMode related variables
//
bool isBlackScreenActivated = false; //Tells us if the PSP has been 'Black Screened'
int currentBrightness; //Stores the current brightness value, from 0 to 100

/**
* toggleMusicMode
*
* Function which underclocks the CPU and the bus of
* the console when requested to a speed suitable for listening
* to your music, when you call again to this function it restores
* the previous frequencies you had.
*
*/
void toggleMusicMode()
{
  sceCtrlPeekBufferPositive(&pad, 1); 
  if (!isMusicMode)
  {
	  clockFrequency = scePowerGetCpuClockFrequencyInt();
	  busFrequency = scePowerGetBusClockFrequency();
	  pspTime currentTime;
	  sceRtcGetCurrentClock(&currentTime,0);
	  int currentSeconds = currentTime.seconds;
	  while(currentTime.seconds < currentSeconds + 1 && !isMusicMode)
	  {
		  if(isMusicMode)
		  {
			  break;
		  }
		  sceRtcGetCurrentClock(&currentTime,0);
		  blit_string(0,0,"Music Mode Enabled");
		  sceDisplayWaitVblankStart();
	  }
	  sceDisplayWaitVblankStart();
	  sctrlHENSetSpeed(MUSIC_MODE_CLKSPD,MUSIC_MODE_BUSSPD);
	  isMusicMode = true;
  }
  else
  {
	sctrlHENSetSpeed(clockFrequency,busFrequency);	
	pspTime currentTime;
	sceRtcGetCurrentClock(&currentTime,0);
	int currentSeconds = currentTime.seconds;
	while(currentTime.seconds < currentSeconds + 1 && isMusicMode)
	{
		if(!isMusicMode)
		{
			  break;
		}
		sceRtcGetCurrentClock(&currentTime,0);
		blit_string(0,0,"Music Mode Disabled");
		sceDisplayWaitVblankStart();
	}
	sceDisplayWaitVblankStart();
	isMusicMode = false;	
  }
}

/*
* powerLock
*
* Locks or unlocks the power switch via software
* in order to prevent accidental power offs.
*/
void powerLock()
{
	if (!isSwitchLocked)
	{
		pspTime currentTime;
		sceRtcGetCurrentClock(&currentTime,0);
		int currentSeconds = currentTime.seconds;
		while(currentTime.seconds < currentSeconds + 1 && !isSwitchLocked)
		{		
			sceRtcGetCurrentClock(&currentTime,0);
			blit_string(0,0,"Power Switch: Locked");
			sceDisplayWaitVblankStart();
		}
		scePowerLock(0);
		isSwitchLocked = true;
	}
	else
	{
		pspTime currentTime;
		sceRtcGetCurrentClock(&currentTime,0);
		int currentSeconds = currentTime.seconds;
		while(currentTime.seconds < currentSeconds + 1 && isSwitchLocked)
		{		
			sceRtcGetCurrentClock(&currentTime,0);
			blit_string(0,0,"Power Switch: Unlocked");
			sceDisplayWaitVblankStart();
		}
		scePowerUnlock(0);
		isSwitchLocked = false;
	}
}
/*
* createPowerTick
*
* Counts to 59 seconds and then, generates a power tick
* this prevents the console shutdown
*/
void createPowerTick()
{
	pspTime currentTime;
	sceRtcGetCurrentClock(&currentTime,0);
	int currentSeconds = currentTime.seconds;
	while(currentTime.seconds < currentSeconds + 59 && isIdleDeactivated)
	{
		//Waits
		sceRtcGetCurrentClock(&currentTime,0);		
		sceCtrlPeekBufferPositive(&pad, 1);
		if(RPRESSED && STARTPRESSED)
		{
			isIdleDeactivated = false;
			scePowerIdleTimerEnable(0);
			sceRtcGetCurrentClock(&currentTime,0);
			currentSeconds = currentTime.seconds;
			while(currentTime.seconds < currentSeconds + 1)
			{		
				sceRtcGetCurrentClock(&currentTime,0);
				blit_string(0,0,"ToggleIdle Off");
				sceDisplayWaitVblankStart();
			}			
		}
		sceKernelDelayThread(10000);
	}
	scePowerTick(0);
}
/*
* toggleIdle
*
* Similar to the functions above, this overrides the idle
* timer of the console, making it 'always on' preventing
* from powering off when you are not interacting with it
*
* Useful when, why hide it, watching pr0n... ;-)
*/
void toggleIdle()
{
	if(!isIdleDeactivated)
	{
		scePowerIdleTimerDisable(0);
		pspTime currentTime;
		sceRtcGetCurrentClock(&currentTime,0);
		int currentSeconds = currentTime.seconds;
		while(currentTime.seconds < currentSeconds + 1 && !isIdleDeactivated)
		{		
			sceRtcGetCurrentClock(&currentTime,0);
			blit_string(0,0,"ToggleIdle On, PSP won't turn off");
			sceDisplayWaitVblankStart();
		}		
		isIdleDeactivated = true;
		while(isIdleDeactivated)
		{
			createPowerTick();
			sceKernelDelayThread(150000);
		}			
	}
	else
	{
		scePowerIdleTimerEnable(0);
		pspTime currentTime;
		sceRtcGetCurrentClock(&currentTime,0);
		int currentSeconds = currentTime.seconds;
		while(currentTime.seconds < currentSeconds + 1 && isIdleDeactivated)
		{		
			sceRtcGetCurrentClock(&currentTime,0);
			blit_string(0,0,"ToggleIdle Off");
			sceDisplayWaitVblankStart();
		}
		isIdleDeactivated = false;
	}
}
/*
* blackScreenMode
*
* This mode turns off completely the PSP display
* this mode must be activated in last place, because
* the other modes can not be activated once Black Screen
* mode has been activated.
*
* You can exit this mode by pressing L + R.
*
*/
void blackScreenMode()
{
	if(isBlackScreenActivated == false)
	{
		isBlackScreenActivated = true;
		sceDisplayGetBrightness(&currentBrightness,0);
		while(isBlackScreenActivated == true)
		{
			sceKernelDelayThread(150000);
			sceDisplaySetBrightness(0,0);
			sceDisplayDisable();
			sceCtrlPeekBufferPositive(&pad, 1);
			if(LPRESSED && STARTPRESSED)
			{
					isBlackScreenActivated = false;
					sceDisplayEnable();
					sceDisplaySetBrightness(currentBrightness,0);					
			}			
			sceKernelDelayThread(100000);
		}
	}
}
/**
* main_thread 
* 
* Plugin's main thread, like EBOOTs main, here go
* all the calls to the functions previously coded.
*
* Unlike the EBOOT's main, this is NOT the entry point
*/
int main_thread(SceSize args, void* argp)
{
	sceKernelDelayThread(3000000);
	blit_setup();
	blit_set_color(FGCOLOR,BGCOLOR);
	while (1) //We are forever people (always wanted to say that since i saw it in ps2sdk samples)
	{
		sceCtrlPeekBufferPositive(&pad, 1);
		if (pad.Buttons != 0)
		{
			if (RPRESSED && LPRESSED)
			{
				toggleMusicMode(); // Music Mode combo
			}					
			if(RPRESSED && NOTEPRESSED)
			{
				powerLock(); // Power switch locker combo
			}		
			if(RPRESSED && STARTPRESSED)
			{
				toggleIdle(); // Idle Timer override combo
			}
			
			if(LPRESSED && SQUAREPRESSED)
			{
				scePowerRequestStandby(); //Triggers PSP Shutdown (Now without crashes!!)
			}
			if(LPRESSED && STARTPRESSED)
			{
				blackScreenMode();
			}						
		}
		sceKernelDelayThread(100000);
	}
	sceKernelExitDeleteThread(0);
    return 0;
}
/**
* module_start 
*
* Entry point to the plugin, this function
* creates the main thread, starts it and then 
* ends, this is only the entry point to the plugin
* 
* The main plugin code is not written here, is in the
* main thread function.
*/
extern "C"{
int module_start(SceSize args, void *argp)
{
	int thid1 = sceKernelCreateThread("powermgr_thread", main_thread, 10, 0x2000, 0, NULL);
	if(thid1 >= 0)
		sceKernelStartThread(thid1, args, argp);
	return 0;
}
}
/**
* module_stop
*
* Terminates the plugin execution
*/
extern "C"{
int module_stop (void) {
	return 0;
}
}
