/**
* Power Management Plugin 0.2.1
* ---------------------------
* Coded by 3r Ch4mp1 and dedicated to Mariloli, the girl of my life
* my unique dream is to kiss you till the sunrise.
* 
* Parts of code are from Anissian's PRX tutorial samples and from DAX
* HEN SDK samples and some other code that I found on pspdev.org forum.
* 
* Hope this crap can be useful to someone, I did only to learn and enjoy
* with C/C++
*
* You are free to use and modify this source code, if you do it, give me credit
* and say that you extracted code from here.
*/

#include <pspsdk.h>
#include <pspkernel.h>
#include <pspctrl.h>
#include <psppower.h>
#include <pspdisplay_kernel.h>
#include <pspsystimer.h>
#include "contrib/blit.h"

#define KERNEL_MODE 0x1000

//Button Pressed defines

#define RPRESSED pad.Buttons & PSP_CTRL_RTRIGGER
#define LPRESSED pad.Buttons & PSP_CTRL_LTRIGGER
#define STARTPRESSED pad.Buttons & PSP_CTRL_START
#define SELECTPRESSED pad.Buttons & PSP_CTRL_SELECT
#define NOTEPRESSED pad.Buttons & PSP_CTRL_NOTE

//VSH Message colours defines

#define FGCOLOR 0xFFFFFF //Foregorund color is pure white
#define BGCOLOR 0x000000 //Background color is pure black

PSP_MODULE_INFO("PowerMgr", KERNEL_MODE, 0, 21);
PSP_MAIN_THREAD_ATTR(0);

/* 
* These are used to access to the functions 
* of pspdisplay_driver, they are equivalent to
* the ones on pspdisplay.h but we are in kernel
* mode and we must use them or the plugin won't
* work
*/
extern "C"{
int sceDisplayEnable(void);
int sceDisplayDisable(void);
int sceDisplayWaitVblankStart(void); 
}

SceCtrlData pad; // Structure which represents the PSP controls, we'll use it to interact with the PSP

SceSysTimerId timer; //Timing structure to handle system clock ticks
bool elapsed; //Boolean which tells us if a second has elapsed, it's only modified by the tick handler

//
//Music Mode related variables
//

bool isMusicMode = false; // Boolean which tells us if we put the PSP in Music Mode

int clockFrequency;
int busFrequency;

//
//Clock & Bus Speed defines
//

#define MUSIC_MODE_CLKSPD 60 
#define MUSIC_MODE_BUSSPD 30  

//
//PowerLock related variables 
//

bool isSwitchLocked = false;

//
//IdleControl related variables
//

bool isIdleDeactivated = false;

//
//BlackScreenMode related variables
//
bool isBlackScreenActivated = false; //Tells us if the PSP has been 'Black Screened'
int currentBrightness; //Stores the current brightness value, from 0 to 100

/*
* This tick handler is used to
* handle the CPU ticks and count exactly
* 1 second
*/
int tickHandler()
{
	elapsed = false;
	static int fraction;
	fraction++;
	if (fraction > 12)
	{
		fraction = 0;
		elapsed = true;
	}
	return -1;
}

/**
* toggleMusicMode
*
* Function which underclocks the CPU and the bus of
* the console when requested to a speed suitable for listening
* to your music, when you call again to this function it restores
* the previous frequencies you had.
*
*/
void toggleMusicMode()
{
  sceCtrlPeekBufferPositive(&pad, 1); 
  if (!isMusicMode)
  {
	  clockFrequency = scePowerGetCpuClockFrequency();
	  busFrequency = scePowerGetBusClockFrequency();
	  scePowerSetClockFrequency(MUSIC_MODE_CLKSPD, MUSIC_MODE_CLKSPD, MUSIC_MODE_BUSSPD);
	  timer = sceSTimerAlloc();
	  sceSTimerSetHandler(timer, 4194303, tickHandler, 0);
	  sceSTimerStartCount(timer);
	  while(!elapsed)
	  {
		  blit_string(0,0,"Music Mode Enabled");
		  sceDisplayWaitVblankStart();
	  }
	  sceSTimerStopCount(timer);
	  sceSTimerResetCount(timer);
	  sceSTimerFree(timer);
	  elapsed = false;
	  isMusicMode = true;	  
  }
  else
  {
	scePowerSetClockFrequency(clockFrequency, clockFrequency, busFrequency);	
	timer = sceSTimerAlloc();
	sceSTimerSetHandler(timer, 4194303, tickHandler, 0);
	sceSTimerStartCount(timer);
	while(!elapsed)
	{
		blit_string(0,0,"Music Mode Disabled");
		sceDisplayWaitVblankStart();
	}
	sceSTimerStopCount(timer);
	sceSTimerResetCount(timer);
	sceSTimerFree(timer);
	elapsed = false;	
	isMusicMode = false;	
  }
}

/*
* powerLock
*
* Locks or unlocks the power switch via software
* in order to prevent accidental power offs.
*/
void powerLock()
{
	if (isSwitchLocked == false)
	{
		scePowerLock(0);
		timer = sceSTimerAlloc();
		sceSTimerSetHandler(timer, 4194303, tickHandler, 0);
		sceSTimerStartCount(timer);
		while(!elapsed)
		{
			blit_string(0,0,"Power Switch: Locked");
			sceDisplayWaitVblankStart();
		}
		sceSTimerStopCount(timer);
		sceSTimerResetCount(timer);
		sceSTimerFree(timer);
		elapsed = false;
		isSwitchLocked = true;
	}
	else
	{
		scePowerUnlock(0);
		timer = sceSTimerAlloc();
		sceSTimerSetHandler(timer, 4194303, tickHandler, 0);
		sceSTimerStartCount(timer);
		while(!elapsed)
		{
			blit_string(0,0,"Power Switch: Unlocked");
			sceDisplayWaitVblankStart();
		}
		sceSTimerStopCount(timer);
		sceSTimerResetCount(timer);
		sceSTimerFree(timer);
		elapsed = false;
		isSwitchLocked = false;
	}
}

/*
* toggleIdle
*
* Similar to the functions above, this overrides the idle
* timer of the console, making it 'always on' preventing
* from powering off when you are not interacting with it
*
* Useful when, why hide it, watching porn... ;-)
*/
void toggleIdle()
{
	if(isIdleDeactivated == false)
	{
		scePowerIdleTimerDisable(0);
		timer = sceSTimerAlloc();
		sceSTimerSetHandler(timer, 4194303, tickHandler, 0);
		sceSTimerStartCount(timer);
		while(!elapsed)
		{
			blit_string(0,0,"ToggleIdle On, PSP won't turn off");
			sceDisplayWaitVblankStart();
		}
		sceSTimerStopCount(timer);
		sceSTimerResetCount(timer);
		sceSTimerFree(timer);
		elapsed = false;
		isIdleDeactivated = true;
	}
	else
	{
		scePowerIdleTimerEnable(0);
		timer = sceSTimerAlloc();
		sceSTimerSetHandler(timer, 4194303, tickHandler, 0);
		sceSTimerStartCount(timer);
		while(!elapsed)
		{
			blit_string(0,0,"ToggleIdle Off");
			sceDisplayWaitVblankStart();
		}
		sceSTimerStopCount(timer);
		sceSTimerResetCount(timer);
		sceSTimerFree(timer);
		elapsed = false;
		isIdleDeactivated = false;
	}
}
/*
* blackScreenMode
*
* This mode turns off completely the PSP display
* this mode must be activated in last place, because
* the other modes can not be activated once Black Screen
* mode has been activated.
*
* You can exit this mode by pressing L + R.
*
*/
void blackScreenMode()
{
	if(isBlackScreenActivated == false)
	{
		isBlackScreenActivated = true;
		sceDisplayGetBrightness(&currentBrightness,0);
		while(isBlackScreenActivated == true)
		{
			sceKernelDelayThread(150000);
			sceDisplaySetBrightness(0,0);
			sceDisplayDisable();
			sceCtrlPeekBufferPositive(&pad, 1);
			if(LPRESSED && STARTPRESSED)
			{
					isBlackScreenActivated = false;
					sceDisplayEnable();
					sceDisplaySetBrightness(currentBrightness,0);					
			}			
			sceKernelDelayThread(100000);
		}
	}
}
/**
* main_thread 
* 
* Plugin's main thread, like EBOOTs main, here go
* all the calls to the functions previously coded.
*
* Unlike the EBOOT's main, this is NOT the entry point
*/
int main_thread(SceSize args, void* argp)
{
	sceKernelDelayThread(3000000);
	blit_setup();
	blit_set_color(FGCOLOR,BGCOLOR);
	while (1) //We are forever people (always wanted to say that since i saw it in ps2sdk samples)
	{
		sceCtrlPeekBufferPositive(&pad, 1);
		if (pad.Buttons != 0)
		{
			if (RPRESSED && LPRESSED)
			{
				toggleMusicMode(); // Music Mode combo
			}					
			if(RPRESSED && NOTEPRESSED)
			{
				powerLock(); // Power switch locker combo
			}		
			if(RPRESSED && STARTPRESSED)
			{
				toggleIdle(); // Idle Timer override combo
			}
			
			if(LPRESSED && SELECTPRESSED)
			{
				scePowerRequestStandby(); //Triggers PSP Shutdown (Now without crashes!!)
			}
			if(LPRESSED && STARTPRESSED)
			{
				blackScreenMode();
			}						
		}
		sceKernelDelayThread(100000);
	}
	sceKernelExitDeleteThread(0);
    return 0;
}
/**
* module_start 
*
* Entry point to the plugin, this function
* creates the main thread, starts it and then 
* ends, this is only the entry point to the plugin
* 
* The main plugin code is not written here, is in the
* main thread function.
*/
extern "C"{
int module_start(SceSize args, void *argp)
{
	int thid1 = sceKernelCreateThread("powermgr_thread", main_thread, 10, 0x2000, 0, NULL);
	if(thid1 >= 0)
		sceKernelStartThread(thid1, args, argp);
	return 0;
}
}
/**
* module_stop
*
* Terminates the plugin execution
*/
extern "C"{
int module_stop (void) {
	return 0;
}
}
